/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: source_table.h,v 1.4 2002/05/08 21:24:48 jdejong Exp $ */
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: source_table.h,v 1.4 2002/05/08 21:24:48 jdejong Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jdejong 2002-05-08 created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef _SOURCE_TABLE_H
#define _SOURCE_TABLE_H
/*----------------------------------------------------------------------- */
/* Source table FITS binary tables routines                               */
/*----------------------------------------------------------------------- */
#include "VLTItable.h"

typedef struct      /* table definition for source table */
{
  table_def *def;       /* basic table information */
  int    nbands;        /* Maximum number of entries in FREQ */
  long   row;           /* last row (0-rel) read or written, -1=> none. */
  int    source_id_col; /* time column number (0-rel)*/
  int    source_col;    /* time interval column number */
  int    calcode_col;   /* source number column number */
  int    iflux_col;     /* iflux column number */
  int    qflux_col;     /* qflux column number */
  int    uflux_col;     /* uflux column number */
  int    vflux_col;     /* vflux column number */
  int    freq_col;      /* freq column number */
  int    raepp_col;     /* raepp column number */
  int    decepp_col;    /* decepp column number */
  int    equinox_col;   /* equinox column number */
  int    rapp_col;      /* rapp column number */
  int    decapp_col;    /* decapp column number */
  int    sysvel_col;    /* sysvel column number */
  int    veltyp_col;    /* veltyp column number */
  int    veldef_col;    /* veldef column number */
  int    pmra_col;      /* pmra column number */
  int    pmdec_col;     /* pmdec column number */
  int    parallax_col;  /* parallax column number */
  int    diameter_col;  /* diameter column number */
  int    diam_err_col;  /* diam_err column number */
} source_table_def;

typedef struct      /* structure used for source_table row */
{
  double raepp;      /* Right ascension at mean epoch (deg) */
  double decepp;     /* Declination at mean epoch  (deg) */
  double rapp;       /* Apparent Right Ascension (deg) */
  double decapp;     /* Apparent Declination (deg) */
  double sysvel;     /* Systemic velocity (m/sec) */
  double pmra;       /* Proper motion in RA (deg/day) */
  double pmdec;      /* Proper motion in dec (deg/day) */
  float parallax;    /* Parallax (arcsecond) */
  float diameter;    /* Source diameter (arcsec) neg -> not known */
  float diam_err;    /* RMS error in diameter */
  int   source_id;   /* source number */
  char  source[18];  /* source name */
  char  calcode[6];  /* calibrator code */
  char equinox[10];  /* Equinox of raepp,decepp (B1950.0 or J2000.0) */
  char veltyp[10];   /* Velocity type of sysvel */
  char veldef[10];   /* velocity definition 'RADIO' or 'OPTICAL' */
  float  *iflux;     /* Total (Stokes' I) flux density (Jy) */
  float  *qflux;     /* Stokes' Q Flux density (Jy) */
  float  *uflux;     /* Stokes' U Flux density (Jy) */
  float  *vflux;     /* Stokes' V Flux density (Jy) */
  double *freq;      /* Frequencies (Hz) corresponding to iflux etc. */
  table_row *raw;    /* generic table row */
} source_table_row;

/* public functions */
/* there are more complete descriptions of the functions in the source file */

/* source_table_row functions */
source_table_row* make_source_table_row(source_table_def* table);
/*----------------------------------------------------------------------- */
/*   Make an source_table_row structure                                   */
/*   Inputs:                                                              */
/*      table     pointer to source_table table structure                 */
/*   Returns:                                                             */
/*      An source_table_row structure pointer ; NULL indicates failure    */
/*      Data pointers set to generic structure values                     */
/*----------------------------------------------------------------------- */

void kill_source_table_row (source_table_row *row);
/*----------------------------------------------------------------------- */
/*   Delete an source_table_row structure                                 */
/*   Inputs:                                                              */
/*      row    pointer to the source_table_row structure.                 */
/*----------------------------------------------------------------------- */

/* source_table functions */
source_table_def* 
make_source_table_def(fitsfile *fptr, int nbands, int *ierr);
/*----------------------------------------------------------------------- */
/*   Make a new source_table in an open FITS file                         */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      nbands     number of values in freq, iflux...                     */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      A source_table_def pointer; NULL indicates failure                */
/*----------------------------------------------------------------------- */

source_table_def* 
open_source_table(fitsfile *fptr, int *ierr);
/*----------------------------------------------------------------------- */
/*   Open an existing source_table in the active FITS file                */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      An source_table_def pointer; NULL indicates failure               */
/*----------------------------------------------------------------------- */

int read_source_table(fitsfile *fptr,  source_table_def* table, 
			    source_table_row *row, long row_number,
			    int *ierr);
/*----------------------------------------------------------------------- */
/*   Read the next row from an source_table                               */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row_number which row (1-rel) in the table                         */   
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      row       pointer to an extant row structure                      */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int write_source_table(fitsfile *fptr, source_table_def *table, 
			     source_table_row *row,  long row_number,
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Write  the next row to an source_table                               */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row       pointer to an extant row structure                      */
/*      row_number which row (1-rel) in the table                         */
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int close_source_table(fitsfile *fptr, source_table_def* table, 
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Close an open source_table, associated structures are destroyed      */
/*   Note: this closes the full FITS file                                 */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

void kill_source_table_def (source_table_def* table);
/*----------------------------------------------------------------------- */
/*   Delete an source_table structure                                     */
/*   Inputs:                                                              */
/*      table     table definitions structure from open or create         */
/*----------------------------------------------------------------------- */

#endif /* end SOURCE_TABLE_H */
/* */
